@@ -20,16 +20,16 @@ import json
 import base64
 import bottle
 
-from IM.InfrastructureInfo import IncorrectVMException, DeletedVMException
+from IM.InfrastructureInfo import IncorrectVMException, DeletedVMException, IncorrectStateException
 from IM.InfrastructureManager import (InfrastructureManager, DeletedInfrastructureException,
                                       IncorrectInfrastructureException, UnauthorizedUserException,
-                                      InvaliddUserException)
+                                      InvaliddUserException, DisabledFunctionException)
 from IM.auth import Authentication
 from IM.config import Config
+from IM import get_ex_error
 from radl.radl_json import parse_radl as parse_radl_json, dump_radl as dump_radl_json, featuresToSimple, radlToSimple
 from radl.radl import RADL, Features, Feature
 from IM.tosca.Tosca import Tosca
-from IM.VirtualMachine import VirtualMachine
 
 logger = logging.getLogger('InfrastructureManager')
 
@@ -68,7 +68,7 @@ class MySSLCherryPy(bottle.ServerAdapter):
             from cheroot.ssl.pyopenssl import pyOpenSSLAdapter
             from cheroot import wsgi
             server = wsgi.Server((self.host, self.port), handler, request_queue_size=32)
-        except:
+        except Exception:
             from cherrypy.wsgiserver.ssl_pyopenssl import pyOpenSSLAdapter
             from cherrypy import wsgiserver
             server = wsgiserver.CherryPyWSGIServer((self.host, self.port), handler, request_queue_size=32)
@@ -96,7 +96,7 @@ class MyCherryPy(bottle.ServerAdapter):
             # First try to use the new version
             from cheroot import wsgi
             server = wsgi.Server((self.host, self.port), handler, request_queue_size=32)
-        except:
+        except Exception:
             from cherrypy import wsgiserver
             server = wsgiserver.CherryPyWSGIServer((self.host, self.port), handler, request_queue_size=32)
 
@@ -203,7 +203,7 @@ def get_auth_header():
     auth_header = bottle.request.headers['AUTHORIZATION']
     if Config.SINGLE_SITE:
         if auth_header.startswith("Basic "):
-            auth_data = base64.b64decode(auth_header[6:])
+            auth_data = str(base64.b64decode(auth_header[6:]))
             user_pass = auth_data.split(":")
             im_auth = {"type": "InfrastructureManager",
                        "username": user_pass[0],
@@ -309,29 +309,53 @@ def format_output(res, default_type="text/plain", field_name=None, list_field_na
 def RESTDestroyInfrastructure(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
-        InfrastructureManager.DestroyInfrastructure(infid, auth)
+        force = False
+        if "force" in bottle.request.params.keys():
+            str_force = bottle.request.params.get("force").lower()
+            if str_force in ['yes', 'true', '1']:
+                force = True
+            elif str_force in ['no', 'false', '0']:
+                force = False
+            else:
+                return return_error(400, "Incorrect value in force parameter")
+
+        async_call = False
+        if "async" in bottle.request.params.keys():
+            str_ctxt = bottle.request.params.get("async").lower()
+            if str_ctxt in ['yes', 'true', '1']:
+                async_call = True
+            elif str_ctxt in ['no', 'false', '0']:
+                async_call = False
+            else:
+                return return_error(400, "Incorrect value in async parameter")
+
+        InfrastructureManager.DestroyInfrastructure(infid, auth, force, async_call)
         bottle.response.content_type = "text/plain"
         return ""
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Destroying Inf: %s" % ex.message)
+        return return_error(404, "Error Destroying Inf: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Destroying Inf: %s" % ex.message)
+        return return_error(404, "Error Destroying Inf: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Destroying Inf: %s" % ex.message)
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+    except IncorrectStateException as ex:
+        return return_error(409, "Error Destroying Inf: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Destroying Inf")
-        return return_error(400, "Error Destroying Inf: %s" % ex.message)
+        return return_error(400, "Error Destroying Inf: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid', method='GET')
 def RESTGetInfrastructureInfo(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -343,21 +367,21 @@ def RESTGetInfrastructureInfo(infid=None):
 
         return format_output(res, "text/uri-list", "uri-list", "uri")
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Getting Inf. info: %s" % ex.message)
+        return return_error(404, "Error Getting Inf. info: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Getting Inf. info: %s" % ex.message)
+        return return_error(404, "Error Getting Inf. info: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Getting Inf. info: %s" % ex.message)
+        return return_error(403, "Error Getting Inf. info: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Getting Inf. info")
-        return return_error(400, "Error Getting Inf. info: %s" % ex.message)
+        return return_error(400, "Error Getting Inf. info: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/:prop', method='GET')
 def RESTGetInfrastructureProperty(infid=None, prop=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -370,7 +394,7 @@ def RESTGetInfrastructureProperty(infid=None, prop=None):
                 elif str_headeronly in ['no', 'false', '0']:
                     headeronly = False
                 else:
-                    return return_error(400, "Incorrect value in context parameter")
+                    return return_error(400, "Incorrect value in headeronly parameter")
 
             res = InfrastructureManager.GetInfrastructureContMsg(infid, auth, headeronly)
         elif prop == "radl":
@@ -429,25 +453,29 @@ def RESTGetInfrastructureProperty(infid=None, prop=None):
 
         return format_output(res, field_name=prop)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Getting Inf. prop: %s" % ex.message)
+        return return_error(404, "Error Getting Inf. prop: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Getting Inf. prop: %s" % ex.message)
+        return return_error(404, "Error Getting Inf. prop: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Getting Inf. prop: %s" % ex.message)
+        return return_error(403, "Error Getting Inf. prop: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Getting Inf. prop")
-        return return_error(400, "Error Getting Inf. prop: %s" % ex.message)
+        return return_error(400, "Error Getting Inf. prop: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures', method='GET')
 def RESTGetInfrastructureList():
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
-        inf_ids = InfrastructureManager.GetInfrastructureList(auth)
+        flt = None
+        if "filter" in bottle.request.params.keys():
+            flt = bottle.request.params.get("filter")
+
+        inf_ids = InfrastructureManager.GetInfrastructureList(auth, flt)
         res = []
 
         for inf_id in inf_ids:
@@ -455,17 +483,17 @@ def RESTGetInfrastructureList():
 
         return format_output(res, "text/uri-list", "uri-list", "uri")
     except InvaliddUserException as ex:
-        return return_error(401, "Error Getting Inf. List: %s" % ex.message)
+        return return_error(401, "Error Getting Inf. List: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Getting Inf. List")
-        return return_error(400, "Error Getting Inf. List: %s" % ex.message)
+        return return_error(400, "Error Getting Inf. List: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures', method='POST')
 def RESTCreateInfrastructure():
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -507,17 +535,19 @@ def RESTCreateInfrastructure():
 
         return format_output(res, "text/uri-list", "uri")
     except InvaliddUserException as ex:
-        return return_error(401, "Error Getting Inf. info: %s" % ex.message)
+        return return_error(401, "Error Getting Inf. info: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Creating Inf.")
-        return return_error(400, "Error Creating Inf.: %s" % ex.message)
+        return return_error(400, "Error Creating Inf.: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures', method='PUT')
 def RESTImportInfrastructure():
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -535,42 +565,44 @@ def RESTImportInfrastructure():
 
         return format_output(res, "text/uri-list", "uri")
     except InvaliddUserException as ex:
-        return return_error(401, "Error Impporting Inf.: %s" % ex.message)
+        return return_error(401, "Error Impporting Inf.: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Impporting Inf.")
-        return return_error(400, "Error Impporting Inf.: %s" % ex.message)
+        return return_error(400, "Error Impporting Inf.: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid', method='GET')
 def RESTGetVMInfo(infid=None, vmid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
         radl = InfrastructureManager.GetVMInfo(infid, vmid, auth)
         return format_output(radl, field_name="radl")
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Getting VM. info: %s" % ex.message)
+        return return_error(404, "Error Getting VM. info: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Getting VM. info: %s" % ex.message)
+        return return_error(404, "Error Getting VM. info: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Getting VM. info: %s" % ex.message)
+        return return_error(403, "Error Getting VM. info: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error Getting VM. info: %s" % ex.message)
+        return return_error(404, "Error Getting VM. info: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error Getting VM. info: %s" % ex.message)
+        return return_error(404, "Error Getting VM. info: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Getting VM info")
-        return return_error(400, "Error Getting VM info: %s" % ex.message)
+        return return_error(400, "Error Getting VM info: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid/:prop', method='GET')
 def RESTGetVMProperty(infid=None, vmid=None, prop=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -587,11 +619,14 @@ def RESTGetVMProperty(infid=None, vmid=None, prop=None):
             if step == 1:
                 url = get_full_url('/infrastructures/' + str(infid) + '/vms/' + str(vmid) + '/command?step=2')
                 auth = sel_inf.auth.getAuthInfo("InfrastructureManager")[0]
-                imuser = auth['username']
-                impass = auth['password']
-                command = ('curl -s -H "Authorization: type = InfrastructureManager; '
-                           'username = %s; password = %s" -H "Accept: text/plain" %s' % (imuser, impass, url))
+                if 'token' in auth:
+                    imauth = "token = %s" % auth['token']
+                else:
+                    imauth = "username = %s; password = %s" % (auth['username'], auth['password'])
+                command = ('curl --insecure -s -H "Authorization: type = InfrastructureManager; %s" '
+                           '-H "Accept: text/plain" %s' % (imauth, url))
 
+                ps_command = "ps aux | grep -v grep | grep 'ssh -N -R'"
                 info = """
                 res="wait"
                 while [ "$res" == "wait" ]
@@ -599,11 +634,17 @@ def RESTGetVMProperty(infid=None, vmid=None, prop=None):
                   res=`%s`
                   if [ "$res" != "wait" ]
                   then
-                    eval "$res"
+                    echo "$res" > /var/tmp/reverse_ssh.sh
+                    chmod a+x /var/tmp/reverse_ssh.sh
+                    /var/tmp/reverse_ssh.sh
+                    if [ "$res" != "true" ]
+                    then
+                      echo "*/1 * * * * root %s || /var/tmp/reverse_ssh.sh" > /etc/cron.d/reverse_ssh
+                    fi
                   else
                     sleep 20
                   fi
-                done""" % command
+                done""" % (command, ps_command)
                 logger.debug("Step 1 command: %s" % info)
             elif step == 2:
                 sel_vm = None
@@ -612,9 +653,9 @@ def RESTGetVMProperty(infid=None, vmid=None, prop=None):
                         sel_vm = vm
                         break
                 if not sel_vm:
-                    # this must never happen
-                    logger.error("Specified vmid in step2 is incorrect!!")
-                    info = None
+                    # it sometimes happen when the VM is in creation state
+                    logger.warn("Specified vmid in step2 is incorrect!!")
+                    info = "wait"
                 else:
                     ssh = sel_vm.get_ssh_ansible_master(retry=False)
 
@@ -647,25 +688,25 @@ def RESTGetVMProperty(infid=None, vmid=None, prop=None):
         else:
             return format_output(info, field_name=prop)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Getting VM. property: %s" % ex.message)
+        return return_error(404, "Error Getting VM. property: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Getting VM. property: %s" % ex.message)
+        return return_error(404, "Error Getting VM. property: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Getting VM. property: %s" % ex.message)
+        return return_error(403, "Error Getting VM. property: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error Getting VM. property: %s" % ex.message)
+        return return_error(404, "Error Getting VM. property: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error Getting VM. property: %s" % ex.message)
+        return return_error(404, "Error Getting VM. property: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Getting VM property")
-        return return_error(400, "Error Getting VM property: %s" % ex.message)
+        return return_error(400, "Error Getting VM property: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid', method='POST')
 def RESTAddResource(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -716,21 +757,23 @@ def RESTAddResource(infid=None):
 
         return format_output(res, "text/uri-list", "uri-list", "uri")
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Adding resources: %s" % ex.message)
+        return return_error(404, "Error Adding resources: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Adding resources: %s" % ex.message)
+        return return_error(404, "Error Adding resources: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Adding resources: %s" % ex.message)
+        return return_error(403, "Error Adding resources: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Adding resources")
-        return return_error(400, "Error Adding resources: %s" % ex.message)
+        return return_error(400, "Error Adding resources: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid', method='DELETE')
 def RESTRemoveResource(infid=None, vmid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -748,25 +791,27 @@ def RESTRemoveResource(infid=None, vmid=None):
         bottle.response.content_type = "text/plain"
         return ""
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error Removing resources: %s" % ex.message)
+        return return_error(404, "Error Removing resources: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error Removing resources: %s" % ex.message)
+        return return_error(404, "Error Removing resources: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error Removing resources: %s" % ex.message)
+        return return_error(403, "Error Removing resources: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error Removing resources: %s" % ex.message)
+        return return_error(404, "Error Removing resources: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error Removing resources: %s" % ex.message)
+        return return_error(404, "Error Removing resources: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error Removing resources")
-        return return_error(400, "Error Removing resources: %s" % ex.message)
+        return return_error(400, "Error Removing resources: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid', method='PUT')
 def RESTAlterVM(infid=None, vmid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -788,25 +833,27 @@ def RESTAlterVM(infid=None, vmid=None):
 
         return format_output(vm_info, field_name="radl")
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error modifying resources: %s" % ex.message)
+        return return_error(404, "Error modifying resources: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error modifying resources: %s" % ex.message)
+        return return_error(404, "Error modifying resources: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error modifying resources: %s" % ex.message)
+        return return_error(403, "Error modifying resources: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error modifying resources: %s" % ex.message)
+        return return_error(404, "Error modifying resources: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error modifying resources: %s" % ex.message)
+        return return_error(404, "Error modifying resources: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error modifying resources")
-        return return_error(400, "Error modifying resources: %s" % ex.message)
+        return return_error(400, "Error modifying resources: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/reconfigure', method='PUT')
 def RESTReconfigureInfrastructure(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -815,7 +862,7 @@ def RESTReconfigureInfrastructure(infid=None):
             str_vm_list = bottle.request.params.get("vm_list")
             try:
                 vm_list = [int(vm_id) for vm_id in str_vm_list.split(",")]
-            except:
+            except Exception:
                 return return_error(400, "Incorrect vm_list format.")
 
         content_type = get_media_type('Content-Type')
@@ -834,106 +881,143 @@ def RESTReconfigureInfrastructure(infid=None):
         bottle.response.content_type = "text/plain"
         return InfrastructureManager.Reconfigure(infid, radl_data, auth, vm_list)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error reconfiguring infrastructure: %s" % ex.message)
+        return return_error(404, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error reconfiguring infrastructure: %s" % ex.message)
+        return return_error(404, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error reconfiguring infrastructure: %s" % ex.message)
+        return return_error(403, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error reconfiguring infrastructure")
-        return return_error(400, "Error reconfiguring infrastructure: %s" % ex.message)
+        return return_error(400, "Error reconfiguring infrastructure: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/start', method='PUT')
 def RESTStartInfrastructure(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
         bottle.response.content_type = "text/plain"
         return InfrastructureManager.StartInfrastructure(infid, auth)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error starting infrastructure: %s" % ex.message)
+        return return_error(404, "Error starting infrastructure: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error starting infrastructure: %s" % ex.message)
+        return return_error(404, "Error starting infrastructure: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error starting infrastructure: %s" % ex.message)
+        return return_error(403, "Error starting infrastructure: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error starting infrastructure")
-        return return_error(400, "Error starting infrastructure: %s" % ex.message)
+        return return_error(400, "Error starting infrastructure: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/stop', method='PUT')
 def RESTStopInfrastructure(infid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
         bottle.response.content_type = "text/plain"
         return InfrastructureManager.StopInfrastructure(infid, auth)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error stopping infrastructure: %s" % ex.message)
+        return return_error(404, "Error stopping infrastructure: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error stopping infrastructure: %s" % ex.message)
+        return return_error(404, "Error stopping infrastructure: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error stopping infrastructure: %s" % ex.message)
+        return return_error(403, "Error stopping infrastructure: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error stopping infrastructure")
-        return return_error(400, "Error stopping infrastructure: %s" % ex.message)
+        return return_error(400, "Error stopping infrastructure: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid/start', method='PUT')
 def RESTStartVM(infid=None, vmid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
         bottle.response.content_type = "text/plain"
         return InfrastructureManager.StartVM(infid, vmid, auth)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error starting VM: %s" % ex.message)
+        return return_error(404, "Error starting VM: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error starting VM: %s" % ex.message)
+        return return_error(404, "Error starting VM: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error starting VM: %s" % ex.message)
+        return return_error(403, "Error starting VM: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error starting VM: %s" % ex.message)
+        return return_error(404, "Error starting VM: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error starting VM: %s" % ex.message)
+        return return_error(404, "Error starting VM: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error starting VM")
-        return return_error(400, "Error starting VM: %s" % ex.message)
+        return return_error(400, "Error starting VM: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid/stop', method='PUT')
 def RESTStopVM(infid=None, vmid=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
         bottle.response.content_type = "text/plain"
         return InfrastructureManager.StopVM(infid, vmid, auth)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error stopping VM: %s" % ex.message)
+        return return_error(404, "Error stopping VM: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error stopping VM: %s" % ex.message)
+        return return_error(404, "Error stopping VM: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error stopping VM: %s" % ex.message)
+        return return_error(403, "Error stopping VM: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error stopping VM: %s" % ex.message)
+        return return_error(404, "Error stopping VM: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error stopping VM: %s" % ex.message)
+        return return_error(404, "Error stopping VM: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error stopping VM")
-        return return_error(400, "Error stopping VM: %s" % ex.message)
+        return return_error(400, "Error stopping VM: %s" % get_ex_error(ex))
+
+
+@app.route('/infrastructures/:infid/vms/:vmid/reboot', method='PUT')
+def RESTRebootVM(infid=None, vmid=None):
+    try:
+        auth = get_auth_header()
+    except Exception:
+        return return_error(401, "No authentication data provided")
+
+    try:
+        bottle.response.content_type = "text/plain"
+        return InfrastructureManager.RebootVM(infid, vmid, auth)
+    except DeletedInfrastructureException as ex:
+        return return_error(404, "Error rebooting VM: %s" % get_ex_error(ex))
+    except IncorrectInfrastructureException as ex:
+        return return_error(404, "Error rebooting VM: %s" % get_ex_error(ex))
+    except UnauthorizedUserException as ex:
+        return return_error(403, "Error rebooting VM: %s" % get_ex_error(ex))
+    except DeletedVMException as ex:
+        return return_error(404, "Error rebooting VM: %s" % get_ex_error(ex))
+    except IncorrectVMException as ex:
+        return return_error(404, "Error rebooting VM: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
+    except Exception as ex:
+        logger.exception("Error rebooting VM")
+        return return_error(400, "Error rebooting VM: %s" % get_ex_error(ex))
 
 
 @app.route('/version', method='GET')
@@ -942,14 +1026,14 @@ def RESTGeVersion():
         from IM import __version__ as version
         return format_output(version, field_name="version")
     except Exception as ex:
-        return return_error(400, "Error getting IM version: %s" % ex.message)
+        return return_error(400, "Error getting IM version: %s" % get_ex_error(ex))
 
 
 @app.route('/infrastructures/:infid/vms/:vmid/disks/:disknum/snapshot', method='PUT')
 def RESTCreateDiskSnapshot(infid=None, vmid=None, disknum=None):
     try:
         auth = get_auth_header()
-    except:
+    except Exception:
         return return_error(401, "No authentication data provided")
 
     try:
@@ -966,24 +1050,26 @@ def RESTCreateDiskSnapshot(infid=None, vmid=None, disknum=None):
             elif str_auto_delete in ['no', 'false', '0']:
                 auto_delete = False
             else:
-                return return_error(400, "Incorrect value in context parameter")
+                return return_error(400, "Incorrect value in auto_delete parameter")
         else:
             auto_delete = False
 
         return InfrastructureManager.CreateDiskSnapshot(infid, vmid, int(disknum), image_name, auto_delete, auth)
     except DeletedInfrastructureException as ex:
-        return return_error(404, "Error creating snapshot: %s" % ex.message)
+        return return_error(404, "Error creating snapshot: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
-        return return_error(404, "Error creating snapshot: %s" % ex.message)
+        return return_error(404, "Error creating snapshot: %s" % get_ex_error(ex))
     except UnauthorizedUserException as ex:
-        return return_error(403, "Error creating snapshot: %s" % ex.message)
+        return return_error(403, "Error creating snapshot: %s" % get_ex_error(ex))
     except DeletedVMException as ex:
-        return return_error(404, "Error creating snapshot: %s" % ex.message)
+        return return_error(404, "Error creating snapshot: %s" % get_ex_error(ex))
     except IncorrectVMException as ex:
-        return return_error(404, "Error creating snapshot: %s" % ex.message)
+        return return_error(404, "Error creating snapshot: %s" % get_ex_error(ex))
+    except DisabledFunctionException as ex:
+        return return_error(403, "Error Destroying Inf: %s" % get_ex_error(ex))
     except Exception as ex:
         logger.exception("Error creating snapshot")
-        return return_error(400, "Error creating snapshot: %s" % ex.message)
+        return return_error(400, "Error creating snapshot: %s" % get_ex_error(ex))
 
 
 @app.error(403)
