@@ -25,13 +25,15 @@ from netaddr import IPNetwork, IPAddress
 
 from radl.radl import network, RADL
 from radl.radl_parse import parse_radl
+from IM.LoggerMixin import LoggerMixin
 from IM.SSH import SSH
 from IM.SSHRetry import SSHRetry
 from IM.config import Config
+from IM import get_user_pass_host_port
 import IM.CloudInfo
 
 
-class VirtualMachine:
+class VirtualMachine(LoggerMixin):
 
     # VM states
     UNKNOWN = "unknown"
@@ -42,6 +44,7 @@ class VirtualMachine:
     FAILED = "failed"
     CONFIGURED = "configured"
     UNCONFIGURED = "unconfigured"
+    DELETING = "deleting"
 
     WAIT_TO_PID = "WAIT"
 
@@ -91,6 +94,8 @@ class VirtualMachine:
         """Flag to specify that this VM is creation process"""
         self.error_msg = None
         """Message with the cause of the the error in the VM (if known) """
+        self.deleting = False
+        """Flag to specify that this VM is deletion process"""
 
     def serialize(self):
         with self._lock:
@@ -154,16 +159,20 @@ class VirtualMachine:
         last = self.is_last_in_cloud(delete_list, remain_vms)
         success = False
         try:
+            self.deleting = True
             VirtualMachine.logger.info("Inf ID: " + self.inf.id + ": Finalizing the VM id: " + str(self.id))
 
             self.kill_check_ctxt_process()
             (success, msg) = self.getCloudConnector().finalize(self, last, auth)
-            if success:
-                self.destroy = True
-            # force the update of the information
-            self.last_update = 0
         except Exception as e:
             msg = str(e)
+        finally:
+            self.deleting = False
+
+        if success:
+            self.destroy = True
+        # force the update of the information
+        self.last_update = 0
 
         if not success:
             VirtualMachine.logger.info("Inf ID: " + self.inf.id + ": The VM cannot be finalized: %s" % msg)
@@ -203,6 +212,15 @@ class VirtualMachine:
         self.last_update = 0
         return (success, msg)
 
+    def reboot(self, auth):
+        """
+        Reboot the VM
+        """
+        (success, msg) = self.getCloudConnector().reboot(self, auth)
+        # force the update of the information
+        self.last_update = 0
+        return (success, msg)
+
     def create_snapshot(self, disk_num, image_name, auto_delete, auth):
         """
         Create a snapshot of one disk of the VM
@@ -257,6 +275,19 @@ class VirtualMachine:
         """
         return self.info.systems[0].getNumNetworkWithConnection(connection)
 
+    def getProxyHost(self):
+        """
+        Return the proxy_host data if available
+        """
+        for netid in self.info.systems[0].getNetworkIDs():
+            net = self.info.get_network_by_id(netid)
+            if net.getValue("proxy_host"):
+                proxy_user, proxy_pass, proxy_ip, proxy_port = get_user_pass_host_port(net.getValue("proxy_host"))
+                if not proxy_port:
+                    proxy_port = 22
+                return SSH(proxy_ip, proxy_user, proxy_pass, net.getValue("proxy_key"), proxy_port)
+        return None
+
     def getIfaceIP(self, iface_num):
         """
         Get the IP of the interface specified
@@ -316,8 +347,7 @@ class VirtualMachine:
         # Or if both VMs are connected to the same network
         i = 0
         while self.info.systems[0].getValue("net_interface." + str(i) + ".connection"):
-            net_name = self.info.systems[0].getValue(
-                "net_interface." + str(i) + ".connection")
+            net_name = self.info.systems[0].getValue("net_interface." + str(i) + ".connection")
 
             common_net = False
             j = 0
@@ -491,29 +521,34 @@ class VirtualMachine:
             if self.state == VirtualMachine.FAILED and self.id is None:
                 return False
 
+            if self.deleting:
+                self.state = VirtualMachine.DELETING
+                return True
+
             now = int(time.time())
             state = self.state
             updated = False
             # To avoid to refresh the information too quickly
             if force or now - self.last_update > Config.VM_INFO_UPDATE_FREQUENCY:
+                success = False
                 try:
                     (success, new_vm) = self.getCloudConnector().updateVMInfo(self, auth)
                     if success:
                         state = new_vm.state
                         updated = True
                         self.last_update = now
-                    elif self.creating:
-                        self.log_info("VM is in creation process, set pending state")
-                        state = VirtualMachine.PENDING
                     else:
                         self.log_error("Error updating VM status: %s" % new_vm)
-                except:
+                except Exception:
                     self.log_exception("Error updating VM status.")
-                    updated = False
+
+                if not success and self.creating:
+                    self.log_info("VM is in creation process, set pending state")
+                    state = VirtualMachine.PENDING
 
             # If we have problems to update the VM info too much time, set to
-            # unknown
-            if now - self.last_update > Config.VM_INFO_UPDATE_ERROR_GRACE_PERIOD:
+            # unknown unless we are still creating the VM
+            if now - self.last_update > Config.VM_INFO_UPDATE_ERROR_GRACE_PERIOD and not self.creating:
                 new_state = VirtualMachine.UNKNOWN
                 self.log_warn("Grace period to update VM info passed. Set state to 'unknown'")
             else:
@@ -564,10 +599,12 @@ class VirtualMachine:
 
         return public_net, num_net
 
-    def setIps(self, public_ips, private_ips, remove_old=False):
+    def setIps(self, public_ips, private_ips, remove_old=False, ignore_nets=None):
         """
         Set the specified IPs in the VM RADL info
         """
+        if not ignore_nets:
+            ignore_nets = []
         vm_system = self.info.systems[0]
 
         # First remove old ip values
@@ -642,27 +679,48 @@ class VirtualMachine:
                         private_net = net
 
                 # Search in the RADL nets, first in the nets this VM is
-                # connected to
+                # connected to and check the CIDR of the nets
                 if private_net is None:
                     for net in self.info.networks:
                         if (not net.isPublic() and net not in private_net_map.values() and
-                                self.getNumNetworkWithConnection(net.id) is not None):
+                                net.id not in ignore_nets and self.getNumNetworkWithConnection(net.id) is not None and
+                                net.getValue('cidr') and IPAddress(private_ip) in IPNetwork(net.getValue('cidr'))):
+                            private_net = net
+                            private_net_map[net.getValue('cidr')] = net
+                            break
+
+                # Now in the RADL nets this VM is connected to
+                # but without CIDR set
+                if private_net is None:
+                    for net in self.info.networks:
+                        if (not net.isPublic() and net not in private_net_map.values() and
+                                net.id not in ignore_nets and self.getNumNetworkWithConnection(net.id) is not None and
+                                not net.getValue('cidr')):
                             private_net = net
                             private_net_map[private_net_mask] = net
                             break
 
                 # Search in the rest of RADL nets
                 if private_net is None:
+                    # First check the CIDR
                     for net in self.info.networks:
-                        if not net.isPublic() and net not in private_net_map.values():
+                        if (not net.isPublic() and net not in private_net_map.values() and net.id not in ignore_nets and
+                                net.getValue('cidr') and IPAddress(private_ip) in IPNetwork(net.getValue('cidr'))):
+                            private_net = net
+                            private_net_map[private_net_mask] = net
+                            break
+
+                    # The search in the rest
+                    for net in self.info.networks:
+                        if (not net.isPublic() and net not in private_net_map.values() and net.id not in ignore_nets and
+                                not net.getValue('cidr')):
                             private_net = net
                             private_net_map[private_net_mask] = net
                             break
 
                 # if it is still None, then create a new one
                 if private_net is None:
-                    private_net = network.createNetwork(
-                        "private." + private_net_mask.split('/')[0])
+                    private_net = network.createNetwork("private." + private_net_mask.split('/')[0])
                     self.info.networks.append(private_net)
                     num_net = self.getNumNetworkIfaces()
                 else:
@@ -683,16 +741,22 @@ class VirtualMachine:
         """
         Get SSH object to connect with this VM
         """
-        (user, passwd, _, private_key) = self.getCredentialValues()
-        ip = self.getPublicIP()
-        if ip is None:
-            ip = self.getPrivateIP()
+        proxy_host = None
+        with self._lock:
+            (user, passwd, _, private_key) = self.getCredentialValues()
+            ip = self.getPublicIP()
+            if ip is None:
+                ip = self.getPrivateIP()
+                if ip and self.getProxyHost():
+                    proxy_host = self.getProxyHost()
+
         if ip is None:
+            self.log_warn("VM ID %s does not have IP. Do not return SSH Object." % self.im_id)
             return None
         if retry:
-            return SSHRetry(ip, user, passwd, private_key, self.getSSHPort())
+            return SSHRetry(ip, user, passwd, private_key, self.getSSHPort(), proxy_host)
         else:
-            return SSH(ip, user, passwd, private_key, self.getSSHPort())
+            return SSH(ip, user, passwd, private_key, self.getSSHPort(), proxy_host)
 
     def is_ctxt_process_running(self):
         """ Return the PID of the running process or None if it is not running """
@@ -714,7 +778,13 @@ class VirtualMachine:
             if self.ctxt_pid != self.WAIT_TO_PID:
                 ssh = self.get_ssh_ansible_master()
                 try:
-                    self.log_info("Killing ctxt process with pid: " + str(self.ctxt_pid))
+                    if not ssh.test_connectivity(5):
+                        self.log_info("Timeout killing ctxt process: %s." % self.ctxt_pid)
+                        self.ctxt_pid = None
+                        self.configured = False
+                        return
+
+                    self.log_info("Killing ctxt process with pid: %s" % self.ctxt_pid)
 
                     # Try to get PGID to kill all child processes
                     pgkill_success = False
@@ -731,7 +801,7 @@ class VirtualMachine:
                                 else:
                                     self.log_error("Error getting PGID of pid: " + str(self.ctxt_pid) +
                                                    ": " + stderr + ". Using only PID.")
-                            except:
+                            except Exception:
                                 self.log_exception("Error getting PGID of pid: " + str(self.ctxt_pid) +
                                                    ": " + stderr + ". Using only PID.")
                         else:
@@ -743,7 +813,7 @@ class VirtualMachine:
 
                     if not pgkill_success:
                         ssh.execute("kill -9 " + str(int(self.ctxt_pid)), 5)
-                except:
+                except Exception:
                     self.log_exception("Error killing ctxt process with pid: " + str(self.ctxt_pid))
 
             self.ctxt_pid = None
@@ -757,15 +827,15 @@ class VirtualMachine:
             self.ctxt_pid = None
             self.configured = False
 
-        ip = self.getPublicIP()
-        if not ip:
-            ip = ip = self.getPrivateIP()
-        remote_dir = Config.REMOTE_CONF_DIR + "/" + \
-            str(self.inf.id) + "/" + ip + "_" + str(self.im_id)
-
         initial_count_out = self.cont_out
         wait = 0
-        while self.ctxt_pid and not self.destroy:
+        while self.ctxt_pid:
+            if self.destroy:
+                # If the VM has been destroyed set pid to None and return
+                self.log_debug("VM %s deleted. Exit check_ctxt_process thread." % self.im_id)
+                self.ctxt_pid = None
+                return None
+
             ctxt_pid = self.ctxt_pid
             if ctxt_pid != self.WAIT_TO_PID:
                 ssh = self.get_ssh_ansible_master()
@@ -773,8 +843,9 @@ class VirtualMachine:
                 try:
                     self.log_info("Getting status of ctxt process with pid: " + str(ctxt_pid))
                     (_, _, exit_status) = ssh.execute("ps " + str(ctxt_pid))
-                except:
-                    self.log_warn("Error getting status of ctxt process with pid: " + str(ctxt_pid))
+                    self.ssh_connect_errors = 0
+                except Exception as ex:
+                    self.log_warn("Error getting status of ctxt process with pid: %s. %s" % (ctxt_pid, ex))
                     exit_status = 0
                     self.ssh_connect_errors += 1
                     if self.ssh_connect_errors > Config.MAX_SSH_ERRORS:
@@ -788,11 +859,16 @@ class VirtualMachine:
                                                              "credentials has been changed.")
                         return None
 
+                ip = self.getPublicIP()
+                if not ip:
+                    ip = ip = self.getPrivateIP()
+                remote_dir = "%s/%s/%s_%s" % (Config.REMOTE_CONF_DIR, self.inf.id, ip, self.im_id)
+
                 if exit_status != 0:
                     # The process has finished, get the outputs
                     self.log_info("The process %s has finished, get the outputs" % ctxt_pid)
-                    ctxt_log = self.get_ctxt_log(remote_dir, True)
-                    msg = self.get_ctxt_output(remote_dir, True)
+                    ctxt_log = self.get_ctxt_log(remote_dir, ssh, True)
+                    msg = self.get_ctxt_output(remote_dir, ssh, True)
                     if ctxt_log:
                         self.cont_out = initial_count_out + msg + ctxt_log
                     else:
@@ -805,7 +881,7 @@ class VirtualMachine:
                     if Config.UPDATE_CTXT_LOG_INTERVAL > 0 and wait > Config.UPDATE_CTXT_LOG_INTERVAL:
                         wait = 0
                         self.log_info("Get the log of the ctxt process with pid: " + str(ctxt_pid))
-                        ctxt_log = self.get_ctxt_log(remote_dir)
+                        ctxt_log = self.get_ctxt_log(remote_dir, ssh)
                         self.cont_out = initial_count_out + ctxt_log
                     # The process is still running, wait
                     self.log_info("The process %s is still running. wait." % ctxt_pid)
@@ -828,14 +904,14 @@ class VirtualMachine:
                 # Otherwise return the value of configured
                 return self.configured
 
-    def get_ctxt_log(self, remote_dir, delete=False):
-        ssh = self.get_ssh_ansible_master()
+    def get_ctxt_log(self, remote_dir, ssh, delete=False):
         tmp_dir = tempfile.mkdtemp()
         conf_out = ""
 
         # Download the contextualization agent log
         try:
             # Get the messages of the contextualization process
+            self.log_debug("Get File: " + remote_dir + '/ctxt_agent.log')
             ssh.sftp_get(remote_dir + '/ctxt_agent.log', tmp_dir + '/ctxt_agent.log')
             with open(tmp_dir + '/ctxt_agent.log') as f:
                 conf_out = f.read()
@@ -846,10 +922,10 @@ class VirtualMachine:
             try:
                 if delete:
                     ssh.sftp_remove(remote_dir + '/ctxt_agent.log')
-            except:
+            except Exception:
                 self.log_exception(
                     "Error deleting remote contextualization process log: " + remote_dir + '/ctxt_agent.log')
-        except:
+        except Exception:
             self.log_exception(
                 "Error getting contextualization process log: " + remote_dir + '/ctxt_agent.log')
             self.configured = False
@@ -858,22 +934,21 @@ class VirtualMachine:
 
         return conf_out
 
-    def get_ctxt_output(self, remote_dir, delete=False):
-        ssh = self.get_ssh_ansible_master()
+    def get_ctxt_output(self, remote_dir, ssh, delete=False):
         tmp_dir = tempfile.mkdtemp()
         msg = ""
 
         # Download the contextualization agent log
         try:
             # Get the JSON output of the ctxt_agent
-            ssh.sftp_get(remote_dir + '/ctxt_agent.out',
-                         tmp_dir + '/ctxt_agent.out')
+            self.log_debug("Get File: " + remote_dir + '/ctxt_agent.out')
+            ssh.sftp_get(remote_dir + '/ctxt_agent.out', tmp_dir + '/ctxt_agent.out')
             with open(tmp_dir + '/ctxt_agent.out') as f:
                 ctxt_agent_out = json.load(f)
             try:
                 if delete:
                     ssh.sftp_remove(remote_dir + '/ctxt_agent.out')
-            except:
+            except Exception:
                 self.log_exception(
                     "Error deleting remote contextualization process output: " + remote_dir + '/ctxt_agent.out')
             # And process it
@@ -897,7 +972,7 @@ class VirtualMachine:
                     stdout += f.read() + "\n"
                 self.log_error(stdout)
                 msg += stdout
-            except:
+            except Exception:
                 self.log_exception("Error getting stdout and stderr to guess why the agent output is not there.")
         except Exception as ex:
             self.log_exception("Error getting contextualization agent output: " + remote_dir + '/ctxt_agent.out')
@@ -948,6 +1023,7 @@ class VirtualMachine:
             if self.inf.vm_master:
                 return self.inf.vm_master.get_ssh(retry=retry)
             else:
+                self.log_warn("There is not master VM. Do not return SSH object.")
                 return None
 
     def __lt__(self, other):
@@ -986,10 +1062,12 @@ class VirtualMachine:
         rest_url = REST_URL if REST_URL else ""
         url = rest_url + '/infrastructures/' + str(self.inf.id) + '/vms/' + str(self.creation_im_id) + '/command'
         auth = self.inf.auth.getAuthInfo("InfrastructureManager")[0]
-        imuser = auth['username']
-        impass = auth['password']
-        command = ('curl -s -H "Authorization: type = InfrastructureManager; '
-                   'username = %s; password = %s" -H "Accept: text/plain" %s' % (imuser, impass, url))
+        if 'token' in auth:
+            imauth = "token = %s" % auth['token']
+        else:
+            imauth = "username = %s; password = %s" % (auth['username'], auth['password'])
+        command = ('curl -s --insecure -H "Authorization: type = InfrastructureManager; %s" '
+                   '-H "Accept: text/plain" %s' % (imauth, url))
         return [command + " | bash &"]
 
     def getSSHReversePort(self):
@@ -1022,21 +1100,26 @@ class VirtualMachine:
 
         return command
 
-    def log_msg(self, level, msg, exc_info=0):
-        msg = "Inf ID: %s: %s" % (self.inf.id, msg)
-        self.logger.log(level, msg, exc_info=exc_info)
+    @staticmethod
+    def delete_public_nets(radl):
+        """
+        Helper function to correctly delete references to public nets in an RADL
+        """
+        nets_id = [net.id for net in radl.networks if net.isPublic()]
+        system = radl.systems[0]
 
-    def log_error(self, msg):
-        self.log_msg(logging.ERROR, msg)
-
-    def log_debug(self, msg):
-        self.log_msg(logging.DEBUG, msg)
-
-    def log_warn(self, msg):
-        self.log_msg(logging.WARNING, msg)
-
-    def log_exception(self, msg):
-        self.log_msg(logging.ERROR, msg, exc_info=1)
-
-    def log_info(self, msg):
-        self.log_msg(logging.INFO, msg)
+        i = 0
+        while system.getValue('net_interface.%d.connection' % i):
+            next_net = system.getValue('net_interface.%d.connection' % (i + 1))
+            next_dns = system.getValue('net_interface.%d.connection' % (i + 1))
+            f = system.getFeature("net_interface.%d.connection" % i)
+            if f.value in nets_id:
+                if next_net:
+                    system.setValue('net_interface.%d.connection' % i, next_net)
+                    system.setValue('net_interface.%d.dns_name' % i, next_dns)
+                else:
+                    system.delValue('net_interface.%d.connection' % i)
+                    system.delValue('net_interface.%d.dns_name' % i)
+                if system.getValue('net_interface.%d.ip' % i):
+                    system.delValue('net_interface.%d.ip' % i)
+            i += 1
