@@ -18,7 +18,10 @@ import uuid
 import random
 import string
 import base64
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from .CloudConnector import CloudConnector
 from radl.radl import Feature
@@ -27,14 +30,13 @@ from IM.config import Config
 try:
     from azure.mgmt.resource import ResourceManagementClient
     from azure.mgmt.storage import StorageManagementClient
-    from azure.storage.blob import BlockBlobService
     from azure.mgmt.compute import ComputeManagementClient
     from azure.mgmt.network import NetworkManagementClient
     from azure.mgmt.dns import DnsManagementClient
     from azure.common.credentials import UserPassCredentials
     from azure.common.credentials import ServicePrincipalCredentials
     from msrestazure.azure_exceptions import CloudError
-    from azure.mgmt.compute.models import DiskCreateOption
+    from azure.mgmt.compute.models import DiskCreateOption, CachingTypes
 except Exception as ex:
     print("WARN: Python Azure SDK not correctly installed. AzureCloudConnector will not work!.")
     print(ex)
@@ -54,6 +56,8 @@ class AzureCloudConnector(CloudConnector):
     """Default instance type."""
     DEFAULT_LOCATION = "westeurope"
     """Default location to use"""
+    DEFAULT_USER = 'azureuser'
+    """ default user to SSH access the VM """
 
     PROVISION_STATE_MAP = {
         'Accepted': VirtualMachine.PENDING,
@@ -115,7 +119,8 @@ class AzureCloudConnector(CloudConnector):
 
         return self.credentials, subscription_id
 
-    def get_instance_type_by_name(self, instance_name, location, credentials, subscription_id):
+    @staticmethod
+    def get_instance_type_by_name(instance_name, location, credentials, subscription_id):
         compute_client = ComputeManagementClient(credentials, subscription_id)
         instace_types = compute_client.virtual_machine_sizes.list(location)
 
@@ -139,23 +144,7 @@ class AzureCloudConnector(CloudConnector):
         if system.getValue('availability_zone'):
             location = system.getValue('availability_zone')
 
-        cpu = 1
-        cpu_op = ">="
-        if system.getFeature('cpu.count'):
-            cpu = system.getValue('cpu.count')
-            cpu_op = system.getFeature('cpu.count').getLogOperator()
-
-        memory = 1
-        memory_op = ">="
-        if system.getFeature('memory.size'):
-            memory = system.getFeature('memory.size').getValue('M')
-            memory_op = system.getFeature('memory.size').getLogOperator()
-
-        disk_free = 0
-        disk_free_op = ">="
-        if system.getValue('disks.free_size'):
-            disk_free = system.getFeature('disks.free_size').getValue('M')
-            disk_free_op = system.getFeature('disks.free_size').getLogOperator()
+        (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(system)
 
         compute_client = ComputeManagementClient(credentials, subscription_id)
         instace_types = list(compute_client.virtual_machine_sizes.list(location))
@@ -166,17 +155,18 @@ class AzureCloudConnector(CloudConnector):
             if instace_type.name == self.INSTANCE_TYPE:
                 default = instace_type
 
-            str_compare = "instace_type.number_of_cores " + cpu_op + " cpu "
-            str_compare += " and instace_type.memory_in_mb " + memory_op + " memory "
-            str_compare += " and instace_type.resource_disk_size_in_mb " + disk_free_op + " disk_free"
+            comparison = cpu_op(instace_type.number_of_cores, cpu)
+            comparison = comparison and memory_op(instace_type.memory_in_mb, memory)
+            comparison = comparison and disk_free_op(instace_type.resource_disk_size_in_mb, disk_free)
 
-            if eval(str_compare):
+            if comparison:
                 if not instance_type_name or instace_type.name == instance_type_name:
                     return instace_type
 
         return default
 
-    def update_system_info_from_instance(self, system, instance_type):
+    @staticmethod
+    def update_system_info_from_instance(system, instance_type):
         """
         Update the features of the system with the information of the instance_type
         """
@@ -189,52 +179,39 @@ class AzureCloudConnector(CloudConnector):
         system.addFeature(Feature("instance_type", "=", instance_type.name),
                           conflict="other", missing="other")
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+
+        if protocol == "azr":
+            credentials, subscription_id = self.get_credentials(auth_data)
+
+            instance_type = self.get_instance_type(radl_system, credentials, subscription_id)
+            if not instance_type:
+                self.log_error("Error generating the RADL of the VM, no instance type available for the requirements.")
+                self.log_debug(radl_system)
+                return None
+
+            res_system = radl_system.clone()
+            username = res_system.getValue('disk.0.os.credentials.username')
+            if not username:
+                res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
+
+            # In Azure we always need to set a password
+            password = res_system.getValue('disk.0.os.credentials.password')
+            if not password:
+                password = ''.join(random.choice(string.ascii_letters + string.digits + "+-*_$@#=<>[]")
+                                   for _ in range(16))
+                res_system.setValue('disk.0.os.credentials.password', password)
+
+            res_system.updateNewCredentialValues()
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
+            return None
 
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-
-                if protocol == "azr":
-                    credentials, subscription_id = self.get_credentials(auth_data)
-
-                    res_system = radl_system.clone()
-                    instance_type = self.get_instance_type(res_system, credentials, subscription_id)
-                    if not instance_type:
-                        self.log_error(
-                            "Error generating the RADL of the VM, no instance type available for the requirements.")
-                        self.log_debug(res_system)
-                    else:
-                        res_system.addFeature(
-                            Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-                        self.update_system_info_from_instance(res_system, instance_type)
-                        res_system.addFeature(
-                            Feature("provider.type", "=", self.type), conflict="other", missing="other")
-
-                        username = res_system.getValue('disk.0.os.credentials.username')
-                        if not username:
-                            res_system.setValue('disk.0.os.credentials.username', 'azureuser')
-
-                        # In Azure we always need to set a password
-                        password = res_system.getValue('disk.0.os.credentials.password')
-                        if not password:
-                            password = ''.join(random.choice(string.ascii_letters + string.digits + "+-*_$@#=<>[]")
-                                               for _ in range(16))
-                            res_system.setValue('disk.0.os.credentials.password', password)
-
-                        res_system.updateNewCredentialValues()
-
-                        res.append(res_system)
-            return res
-
-    def get_rg(self, group_name, credentials, subscription_id):
+    @staticmethod
+    def get_rg(group_name, credentials, subscription_id):
         """
         Get the RG named group_name, if it not exists return None
         """
@@ -247,7 +224,8 @@ class AzureCloudConnector(CloudConnector):
             else:
                 raise cex
 
-    def get_storage_account(self, group_name, storage_name, credentials, subscription_id):
+    @staticmethod
+    def get_storage_account(group_name, storage_name, credentials, subscription_id):
         """
         Get the Storage Account named storage_name in group_name, if it not exists return None
         """
@@ -307,7 +285,7 @@ class AzureCloudConnector(CloudConnector):
         ngs = None
         try:
             ngs = network_client.network_security_groups.create_or_update(group_name, nsg_name, params).result()
-        except:
+        except Exception:
             self.log_exception("Error creating NGS")
 
         return ngs
@@ -403,17 +381,18 @@ class AzureCloudConnector(CloudConnector):
 
         return res
 
-    def get_azure_vm_create_json(self, storage_account, vm_name, nics, radl, instance_type,
-                                 custom_data, compute_client):
+    def get_azure_vm_create_json(self, group_name, storage_account, vm_name, nics, radl,
+                                 instance_type, custom_data, compute_client):
         """ Create the VM parameters structure. """
         system = radl.systems[0]
-        url = uriparse(system.getValue("disk.0.image.url"))
+        url = urlparse(system.getValue("disk.0.image.url"))
         # the url has to have the format: azr://publisher/offer/sku/version
         # azr://Canonical/UbuntuServer/16.04.0-LTS/latest
         # azr://MicrosoftWindowsServerEssentials/WindowsServerEssentials/WindowsServerEssentials/latest
         image_values = (url[1] + url[2]).split("/")
-        if len(image_values) != 4:
-            raise Exception("The Azure image has to have the format: azr://publisher/offer/sku/version")
+        if len(image_values) not in [3, 4]:
+            raise Exception("The Azure image has to have the format: azr://publisher/offer/sku/version"
+                            " or azr://[snapshots|disk]/rgname/diskname")
 
         location = self.DEFAULT_LOCATION
         if system.getValue('availability_zone'):
@@ -428,37 +407,71 @@ class AzureCloudConnector(CloudConnector):
 
         vm = {
             'location': location,
-            'os_profile': {
-                'computer_name': vm_name,
-                'admin_username': user_credentials.username,
-                'admin_password': user_credentials.password,
-                'custom_data': custom_data
-            },
             'hardware_profile': {
                 'vm_size': instance_type.name
             },
-            'storage_profile': {
-                'image_reference': {
-                    'publisher': image_values[0],
-                    'offer': image_values[1],
-                    'sku': image_values[2],
-                    'version': image_values[3]
-                },
-            },
             'network_profile': {
                 'network_interfaces': [{'id': nic.id, 'primary': primary} for nic, primary in nics]
             },
         }
 
-        tags = {}
-        if system.getValue('instance_tags'):
-            keypairs = system.getValue('instance_tags').split(",")
-            for keypair in keypairs:
-                parts = keypair.split("=")
-                key = parts[0].strip()
-                value = parts[1].strip()
-                tags[key] = value
+        if len(image_values) == 3:
+            os_disk_name = "osdisk-" + str(uuid.uuid1())
+            if image_values[0] == "snapshot":
+                managed_disk = compute_client.snapshots.get(image_values[1], image_values[2])
+            elif image_values[0] == "disk":
+                managed_disk = compute_client.disks.get(image_values[1], image_values[2])
+            else:
+                raise Exception("Incorrect image url: it must be snapshot or disk.")
 
+            async_creation = compute_client.disks.create_or_update(
+                group_name,
+                os_disk_name,
+                {
+                    'location': location,
+                    'creation_data': {
+                        'create_option': DiskCreateOption.copy,
+                        'source_resource_id': managed_disk.id
+                    }
+                }
+            )
+
+            os_type = system.getValue("disk.0.os.name")
+            os_type = os_type if os_type else "Linux"
+            self.log_info("Creating OS disk %s of type %s from disk: %s/%s/%s." % (os_disk_name,
+                                                                                   os_type,
+                                                                                   image_values[0],
+                                                                                   image_values[1],
+                                                                                   image_values[2]))
+            disk_resource = async_creation.result()
+
+            vm['storage_profile'] = {
+                'os_disk': {
+                    'create_option': DiskCreateOption.attach,
+                    'os_type': os_type,
+                    'caching': CachingTypes.read_write,
+                    'managed_disk': {
+                        'id': disk_resource.id
+                    }
+                }
+            }
+        else:
+            vm['storage_profile'] = {
+                'image_reference': {
+                    'publisher': image_values[0],
+                    'offer': image_values[1],
+                    'sku': image_values[2],
+                    'version': image_values[3]
+                }
+            }
+            vm['os_profile'] = {
+                'computer_name': vm_name,
+                'admin_username': user_credentials.username,
+                'admin_password': user_credentials.password,
+                'custom_data': custom_data
+            }
+
+        tags = self.get_instance_tags(system)
         if tags:
             vm['tags'] = tags
 
@@ -495,7 +508,7 @@ class AzureCloudConnector(CloudConnector):
 
         return vm
 
-    def create_nets(self, radl, credentials, subscription_id, group_name):
+    def create_nets(self, radl, credentials, subscription_id, group_name, inf):
         network_client = NetworkManagementClient(credentials, subscription_id)
         location = self.DEFAULT_LOCATION
         if radl.systems[0].getValue('availability_zone'):
@@ -508,6 +521,7 @@ class AzureCloudConnector(CloudConnector):
             pass
 
         if not vnet:
+            vnet_cird = self.get_nets_common_cird(radl)
             # Create VNet in the RG of the Inf
             async_vnet_creation = network_client.virtual_networks.create_or_update(
                 group_name,
@@ -515,23 +529,30 @@ class AzureCloudConnector(CloudConnector):
                 {
                     'location': location,
                     'address_space': {
-                        'address_prefixes': ['10.0.0.0/16']
+                        'address_prefixes': [vnet_cird]
                     }
                 }
             )
             async_vnet_creation.wait()
 
             subnets = {}
-            for i, net in enumerate(radl.networks):
+            used_cidrs = []
+            for net in radl.networks:
                 subnet_name = net.id
+                net_cidr = self.get_free_cidr(net.getValue('cidr'), used_cidrs, inf)
+                used_cidrs.append(net_cidr)
+
                 # Create Subnet in the RG of the Inf
                 async_subnet_creation = network_client.subnets.create_or_update(
                     group_name,
                     "privates",
                     subnet_name,
-                    {'address_prefix': '10.0.%d.0/24' % i}
+                    {'address_prefix': net_cidr}
                 )
                 subnets[net.id] = async_subnet_creation.result()
+                net.setValue('cidr', net_cidr)
+                # Set also the cidr in the inf RADL
+                inf.radl.get_network_by_id(net.id).setValue('cidr', net_cidr)
         else:
             subnets = {}
             for i, net in enumerate(radl.networks):
@@ -583,7 +604,7 @@ class AzureCloudConnector(CloudConnector):
 
                 custom_data = self.get_cloud_init_data(radl, vm)
                 instance_type = self.get_instance_type(radl.systems[0], credentials, subscription_id)
-                vm_parameters = self.get_azure_vm_create_json(storage_account_name, vm_name,
+                vm_parameters = self.get_azure_vm_create_json(group_name, storage_account_name, vm_name,
                                                               nics, radl, instance_type, custom_data,
                                                               compute_client)
 
@@ -624,8 +645,19 @@ class AzureCloudConnector(CloudConnector):
 
         credentials, subscription_id = self.get_credentials(auth_data)
 
-        resource_client = ResourceManagementClient(credentials, subscription_id)
+        url = urlparse(radl.systems[0].getValue("disk.0.image.url"))
+        # the url has to have the format: azr://publisher/offer/sku/version or
+        # azr://[snapshots|disk|image]/rgname/diskname
+        # azr://Canonical/UbuntuServer/16.04.0-LTS/latest
+        # azr://MicrosoftWindowsServerEssentials/WindowsServerEssentials/WindowsServerEssentials/latest
+        image_values = (url[1] + url[2]).split("/")
+        if len(image_values) not in [3, 4]:
+            raise Exception("The Azure image has to have the format: azr://publisher/offer/sku/version"
+                            " or azr://[snapshots|disk|image]/rgname/diskname")
+        if len(image_values) == 3 and image_values[0] not in ["snapshot", "disk"]:
+            raise Exception("Incorrect image url: it must be snapshot or disk.")
 
+        resource_client = ResourceManagementClient(credentials, subscription_id)
         storage_account_name = self.get_storage_account_name(inf.id)
 
         with inf._lock:
@@ -648,11 +680,11 @@ class AzureCloudConnector(CloudConnector):
                                                             'kind': 'storage',
                                                             'location': location}
                                                            ).wait()
-                except:
+                except Exception:
                     self.log_exception("Error creating storage account: %s" % storage_account)
                     self.delete_resource_group("rg-%s" % inf.id, resource_client)
 
-            subnets = self.create_nets(radl, credentials, subscription_id, "rg-%s" % inf.id)
+            subnets = self.create_nets(radl, credentials, subscription_id, "rg-%s" % inf.id, inf)
 
         res = []
         vms = self.create_vms(inf, radl, requested_radl, num_vm, location,
@@ -674,7 +706,7 @@ class AzureCloudConnector(CloudConnector):
                     try:
                         group_name = vm.id.split('/')[0]
                         self.delete_resource_group(group_name, resource_client)
-                    except:
+                    except Exception:
                         self.log_exception("Error removing resource group: %s." % group_name)
 
                     res.append((False, "Error waiting the VM %s: %s" % (vm.id, str(ex))))
@@ -775,7 +807,8 @@ class AzureCloudConnector(CloudConnector):
             self.log_exception("Error creating DNS entries")
             return False
 
-    def setIPs(self, vm, network_profile, credentials, subscription_id):
+    @staticmethod
+    def setIPs(vm, network_profile, credentials, subscription_id):
         """
         Set the information about the IPs of the VM
         """
@@ -838,28 +871,29 @@ class AzureCloudConnector(CloudConnector):
         return True, ""
 
     def stop(self, vm, auth_data):
-        try:
-            group_name = vm.id.split('/')[0]
-            vm_name = vm.id.split('/')[1]
-            credentials, subscription_id = self.get_credentials(auth_data)
-            compute_client = ComputeManagementClient(credentials, subscription_id)
-            compute_client.virtual_machines.power_off(group_name, vm_name)
-        except Exception as ex:
-            self.log_exception("Error stopping the VM")
-            return False, "Error stopping the VM: " + str(ex)
-
-        return True, ""
+        return self.vm_action(vm, 'stop', auth_data)
 
     def start(self, vm, auth_data):
+        return self.vm_action(vm, 'start', auth_data)
+
+    def reboot(self, vm, auth_data):
+        return self.vm_action(vm, 'reboot', auth_data)
+
+    def vm_action(self, vm, action, auth_data):
         try:
             group_name = vm.id.split('/')[0]
             vm_name = vm.id.split('/')[1]
             credentials, subscription_id = self.get_credentials(auth_data)
             compute_client = ComputeManagementClient(credentials, subscription_id)
-            compute_client.virtual_machines.start(group_name, vm_name)
+            if action == 'stop':
+                compute_client.virtual_machines.power_off(group_name, vm_name)
+            elif action == 'start':
+                compute_client.virtual_machines.start(group_name, vm_name)
+            elif action == 'reboot':
+                compute_client.virtual_machines.restart(group_name, vm_name)
         except Exception as ex:
-            self.log_exception("Error starting the VM")
-            return False, "Error starting the VM: " + str(ex)
+            self.log_exception("Error restarting the VM")
+            return False, "Error restarting the VM: " + str(ex)
 
         return True, ""
 
