@@ -21,7 +21,10 @@ import os
 import uuid
 import tempfile
 from IM.xmlobject import XMLObject
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from .CloudConnector import CloudConnector
 from radl.radl import UserPassCredential, Feature
@@ -119,6 +122,8 @@ class AzureClassicCloudConnector(CloudConnector):
     """Default location to use"""
     ROLE_NAME = "IMVMRole"
     """Name of the Role"""
+    DEFAULT_USER = 'azureuser'
+    """ default user to SSH access the VM """
 
     DEPLOY_STATE_MAP = {
         'Running': VirtualMachine.RUNNING,
@@ -159,47 +164,32 @@ class AzureClassicCloudConnector(CloudConnector):
 
         return resp
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+
+        if protocol == "azr":
+            instance_type = self.get_instance_type(radl_system, auth_data)
+            if not instance_type:
+                self.log_error("Error generating the RADL of the VM, no instance type available for the requirements.")
+                self.log_debug(radl_system)
+                return None
+
+            res_system = radl_system.clone()
+            self.update_system_info_from_instance(res_system, instance_type)
+
+            username = res_system.getValue('disk.0.os.credentials.username')
+            if not username:
+                res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
+
+            res_system.updateNewCredentialValues()
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
+            return None
 
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-
-                protocol = url[0]
-                if protocol == "azr":
-                    res_system = radl_system.clone()
-                    instance_type = self.get_instance_type(res_system, auth_data)
-                    if not instance_type:
-                        self.log_error(
-                            "Error generating the RADL of the VM, no instance type available for the requirements.")
-                        self.log_debug(res_system)
-                    else:
-                        res_system.addFeature(
-                            Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-                        self.update_system_info_from_instance(
-                            res_system, instance_type)
-                        res_system.addFeature(
-                            Feature("provider.type", "=", self.type), conflict="other", missing="other")
-
-                        username = res_system.getValue(
-                            'disk.0.os.credentials.username')
-                        if not username:
-                            res_system.setValue(
-                                'disk.0.os.credentials.username', 'azureuser')
-
-                        res_system.updateNewCredentialValues()
-
-                        res.append(res_system)
-            return res
-
-    def gen_input_endpoints(self, radl):
+    @staticmethod
+    def gen_input_endpoints(radl):
         """
         Gen the InputEndpoints part of the XML of the VM creation
         using the outports field of the RADL network
@@ -260,7 +250,8 @@ class AzureClassicCloudConnector(CloudConnector):
         res += "\n          </InputEndpoints>"
         return res
 
-    def gen_configuration_set(self, hostname, system):
+    @staticmethod
+    def gen_configuration_set(hostname, system):
         """
         Gen the ConfigurationSet part of the XML of the VM creation
         """
@@ -316,7 +307,8 @@ class AzureClassicCloudConnector(CloudConnector):
 
         return ConfigurationSet
 
-    def gen_data_disks(self, system, storage_account):
+    @staticmethod
+    def gen_data_disks(system, storage_account):
         """
         Gen the DataVirtualHardDisks part of the XML of the VM creation
         """
@@ -352,7 +344,7 @@ class AzureClassicCloudConnector(CloudConnector):
             name = system.getValue("disk.0.image.name")
         if not name:
             name = "userimage" + str(num)
-        url = uriparse(system.getValue("disk.0.image.url"))
+        url = urlparse(system.getValue("disk.0.image.url"))
 
         label = name + " IM created VM"
         (hostname, _) = vm.getRequestedName(
@@ -739,23 +731,7 @@ class AzureClassicCloudConnector(CloudConnector):
         """
         instance_type_name = system.getValue('instance_type')
 
-        cpu = 1
-        cpu_op = ">="
-        if system.getFeature('cpu.count'):
-            cpu = system.getValue('cpu.count')
-            cpu_op = system.getFeature('cpu.count').getLogOperator()
-
-        memory = 1
-        memory_op = ">="
-        if system.getFeature('memory.size'):
-            memory = system.getFeature('memory.size').getValue('M')
-            memory_op = system.getFeature('memory.size').getLogOperator()
-
-        disk_free = 0
-        disk_free_op = ">="
-        if system.getValue('disks.free_size'):
-            disk_free = system.getFeature('disks.free_size').getValue('M')
-            disk_free_op = system.getFeature('disks.free_size').getLogOperator()
+        (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op) = self.get_instance_selectors(system)
 
         instace_types = self.get_all_instance_types(auth_data)
 
@@ -763,17 +739,12 @@ class AzureClassicCloudConnector(CloudConnector):
         for instace_type in instace_types:
             # get the instance type with the lowest Memory
             if res is None or (instace_type.MemoryInMb <= res.MemoryInMb):
-                str_compare = "instace_type.Cores " + cpu_op + " cpu "
-                str_compare += " and instace_type.MemoryInMb " + memory_op + " memory "
-                str_compare += " and instace_type.VirtualMachineResourceDiskSizeInMb " + \
-                    disk_free_op + " disk_free"
 
-                # if arch in instace_type.cpu_arch and
-                # instace_type.cores_per_cpu * instace_type.num_cpu >= cpu and
-                # instace_type.mem >= memory and instace_type.cpu_perf >=
-                # performance and instace_type.disks * instace_type.disk_space
-                # >= disk_free:
-                if eval(str_compare):
+                comparison = cpu_op(instace_type.Cores, cpu)
+                comparison = comparison and memory_op(instace_type.MemoryInMb, memory)
+                comparison = comparison and disk_free_op(instace_type.VirtualMachineResourceDiskSizeInMb, disk_free)
+
+                if comparison:
                     if not instance_type_name or instace_type.Name == instance_type_name:
                         res = instace_type
 
@@ -828,7 +799,7 @@ class AzureClassicCloudConnector(CloudConnector):
                                                VirtualMachine.UNKNOWN)
             else:
                 return self.DEPLOY_STATE_MAP.get(vm_info.Status, VirtualMachine.UNKNOWN)
-        except:
+        except Exception:
             return self.DEPLOY_STATE_MAP.get(vm_info.Status, VirtualMachine.UNKNOWN)
 
     def setIPs(self, vm, vm_info):
@@ -840,18 +811,19 @@ class AzureClassicCloudConnector(CloudConnector):
 
         try:
             role_instance = vm_info.RoleInstanceList.RoleInstance[0]
-        except:
+        except Exception as ex:
+            self.log_debug("%s" % str(ex))
             return
         try:
             if role_instance.IpAddress:
                 private_ips.append(role_instance.IpAddress)
-        except:
-            pass
+        except Exception as ex:
+            self.log_debug("%s" % str(ex))
         try:
             public_ips.append(
                 role_instance.InstanceEndpoints.InstanceEndpoint[0].Vip)
-        except:
-            pass
+        except Exception as ex:
+            self.log_debug("%s" % str(ex))
 
         vm.setIps(public_ips, private_ips)
 
