@@ -1,13 +1,35 @@
+# IM - Infrastructure Manager
+# Copyright (C) 2011 - GRyCAP - Universitat Politecnica de Valencia
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import logging
-import subprocess
-import shutil
-import tempfile
+import operator
 import time
 import yaml
+
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives import serialization
+
+from radl.radl import Feature
 from IM.config import Config
+from IM.LoggerMixin import LoggerMixin
+from netaddr import IPNetwork, spanning_cidr
 
 
-class CloudConnector:
+class CloudConnector(LoggerMixin):
     """
     Base class to all the Cloud connectors
 
@@ -15,6 +37,12 @@ class CloudConnector:
             - cloud_info(:py:class:`IM.CloudInfo`): Data about the Cloud Provider
     """
 
+    OPERATORSMAP = {"<": operator.lt, "<=": operator.le, "=": operator.eq,
+                    ">=": operator.ge, ">": operator.gt, "==": operator.eq}
+    type = "BaseClass"
+    """str with the name of the provider."""
+    DEFAULT_NET_CIDR = "10.0.*.0/24"
+
     def __init__(self, cloud_info, inf):
         self.cloud = cloud_info
         """Data about the Cloud Provider."""
@@ -27,12 +55,19 @@ class CloudConnector:
         self.verify_ssl = Config.VERIFI_SSL
         """Verify SSL connections """
         if not self.verify_ssl:
+            # To avoid errors with host certificates
+            try:
+                import ssl
+                ssl._create_default_https_context = ssl._create_unverified_context
+            except Exception:
+                pass
+
             try:
                 # To avoid annoying InsecureRequestWarning messages in some Connectors
                 import requests.packages
                 from requests.packages.urllib3.exceptions import InsecureRequestWarning
                 requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
-            except:
+            except Exception:
                 pass
 
     def concreteSystem(self, radl_system, auth_data):
@@ -46,8 +81,46 @@ class CloudConnector:
 
         Returns(list of system): list of compatible systems.
         """
+        image_urls = radl_system.getValue("disk.0.image.url")
+        if not image_urls:
+            return [radl_system.clone()]
+        else:
+            if not isinstance(image_urls, list):
+                image_urls = [image_urls]
 
-        return [radl_system.clone()]
+            res = []
+
+            for str_url in image_urls:
+                res_system = self.concrete_system(radl_system, str_url, auth_data)
+                if res_system:
+                    res_system.addFeature(Feature("disk.0.image.url", "=", str_url),
+                                          conflict="other", missing="other")
+                    res_system.addFeature(Feature("provider.type", "=", self.type),
+                                          conflict="other", missing="other")
+                    if self.cloud.server:
+                        res_system.addFeature(Feature("provider.host", "=", self.cloud.server),
+                                              conflict="other", missing="other")
+                    if self.cloud.port != -1:
+                        res_system.addFeature(Feature("provider.port", "=", self.cloud.port),
+                                              conflict="other", missing="other")
+                    res.append(res_system)
+
+            return res
+
+    def concrete_system(self, radl_system, str_url, auth_data):
+        """
+        Return a list of compatible systems with the cloud
+
+        Arguments:
+
+           - radl_system(:py:class:`radl.system`): a system.
+           - str_url(string): a VMI url
+           - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+
+        Returns(:py:class:`radl.system`): a compatible system or none if the url is
+        not compatible with the provider.
+        """
+        return radl_system.clone()
 
     def updateVMInfo(self, vm, auth_data):
         """
@@ -203,6 +276,22 @@ class CloudConnector:
 
         raise NotImplementedError("Should have implemented this")
 
+    def reboot(self, vm, auth_data):
+        """ Reboots a VM
+
+                Arguments:
+                - vm(:py:class:`IM.VirtualMachine`): VM to stop.
+                - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+
+                Returns: a tuple (success, vm).
+           - The first value is True if the operation finished successfully or false otherwise.
+           - The second value is a str with the ID of the stopped VM if the operation finished successfully
+             or an error message otherwise.
+
+        """
+
+        raise NotImplementedError("Should have implemented this")
+
     def create_snapshot(self, vm, disk_num, image_name, auto_delete, auth_data):
         """
         Create a snapshot of the specified num disk in a virtual machine.
@@ -238,36 +327,24 @@ class CloudConnector:
         """
         raise NotImplementedError("Should have implemented this")
 
-    def keygen(self):
+    @staticmethod
+    def keygen():
         """
-        Generates a keypair using the ssh-keygen command and returns a tuple (public, private)
+        Generates a keypair using the cryptography lib and returns a tuple (public, private)
         """
-        tmp_dir = tempfile.mkdtemp()
-        pk_file = tmp_dir + "/im-ssh-key"
-        command = ['ssh-keygen', '-t', 'rsa', '-b', '2048', '-q', '-N', '', '-f', pk_file]
-        p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        (out, err) = p.communicate()
-        if p.returncode != 0:
-            shutil.rmtree(tmp_dir, ignore_errors=True)
-            self.log_error("Error executing ssh-keygen: " + out + err)
-            return (None, None)
-        else:
-            public = None
-            private = None
-            try:
-                with open(pk_file) as f:
-                    private = f.read().strip()
-            except:
-                self.log_exception("Error reading private_key file.")
+        key = rsa.generate_private_key(public_exponent=65537, key_size=2048,
+                                       backend=default_backend())
 
-            try:
-                with open(pk_file + ".pub") as f:
-                    public = f.read().strip()
-            except:
-                self.log_exception("Error reading public_key file.")
+        private = key.private_bytes(encoding=serialization.Encoding.PEM,
+                                    format=serialization.PrivateFormat.TraditionalOpenSSL,
+                                    encryption_algorithm=serialization.NoEncryption()
+                                    ).decode()
 
-            shutil.rmtree(tmp_dir, ignore_errors=True)
-            return (public, private)
+        public = key.public_key().public_bytes(encoding=serialization.Encoding.OpenSSH,
+                                               format=serialization.PublicFormat.OpenSSH
+                                               ).decode()
+
+        return (public, private)
 
     def delete_snapshots(self, vm, auth_data):
         """
@@ -279,8 +356,10 @@ class CloudConnector:
                 success, msg = self.delete_image(image_url, auth_data)
                 if not success:
                     self.log_error("Error deleting snapshot: %s" % msg)
-        except:
+                return success, msg
+        except Exception as ex:
             self.log_exception("Error deleting snapshots.")
+            return success, str(ex)
 
     def get_cloud_init_data(self, radl=None, vm=None, public_key=None, user=None):
         """
@@ -299,6 +378,11 @@ class CloudConnector:
 
             if configure_name:
                 cloud_config = yaml.safe_load(radl.get_configure_by_name(configure_name).recipes)
+                if not isinstance(cloud_config, dict):
+                    # The cloud_init data is a shell script
+                    cloud_config = radl.get_configure_by_name(configure_name).recipes.strip()
+                    self.log_debug(cloud_config)
+                    return cloud_config
 
         # Only for those VMs with private IP
         if Config.SSH_REVERSE_TUNNELS and vm and not vm.hasPublicNet():
@@ -311,12 +395,20 @@ class CloudConnector:
                 cloud_config['bootcmd'] = []
             cloud_config['bootcmd'].extend(curl_command)
 
+        if vm and vm.getSSHPort() != 22:
+            if 'bootcmd' not in cloud_config:
+                cloud_config['bootcmd'] = []
+            cloud_config['bootcmd'].append("sed -i '/Port 22/c\\Port %s' /etc/ssh/sshd_config" % vm.getSSHPort())
+            cloud_config['bootcmd'].append("service sshd restart")
+
         if public_key:
             user_data = {}
             user_data['name'] = user
             user_data['sudo'] = "ALL=(ALL) NOPASSWD:ALL"
             user_data['lock-passwd'] = True
             user_data['ssh-import-id'] = user
+            # avoid to use default /home dir
+            user_data['homedir'] = "/opt/%s" % user
             user_data['ssh-authorized-keys'] = [public_key.strip()]
             if 'users' not in cloud_config:
                 cloud_config['users'] = []
@@ -331,21 +423,128 @@ class CloudConnector:
         else:
             return None
 
-    def log_msg(self, level, msg, exc_info=0):
-        msg = "Inf ID: %s: %s" % (self.inf.id, msg)
-        self.logger.log(level, msg, exc_info=exc_info)
+    @staticmethod
+    def get_instance_tags(system):
+        """
+        Get the instance_tags value of the system object as a dict
+        """
+        tags = {}
+        if system.getValue('instance_tags'):
+            keypairs = system.getValue('instance_tags').split(",")
+            for keypair in keypairs:
+                parts = keypair.split("=")
+                key = parts[0].strip()
+                value = parts[1].strip()
+                tags[key] = value
+        return tags
 
-    def log_error(self, msg):
-        self.log_msg(logging.ERROR, msg)
+    @staticmethod
+    def get_nets_common_cird(radl):
+        """
+        Get a common CIDR in all the RADL nets
+        """
+        nets = []
+        for num, net in enumerate(radl.networks):
+            provider_id = net.getValue('provider_id')
+            if net.getValue('create') == 'yes' and not net.isPublic() and not provider_id:
+                net_cidr = net.getValue('cidr')
+                if not net_cidr:
+                    net_cidr = CloudConnector.DEFAULT_NET_CIDR
+                net_cidr_0 = IPNetwork(net_cidr.replace("*", "0"))
+                if net_cidr_0 not in nets:
+                    nets.append(net_cidr_0)
+                net_cidr = IPNetwork(net_cidr.replace("*", str(num + 1)))
+                nets.append(net_cidr)
 
-    def log_debug(self, msg):
-        self.log_msg(logging.DEBUG, msg)
+        if len(nets) == 0:  # there is no CIDR return the default one
+            return "10.0.0.0/16"
+        elif len(nets) == 1:  # there is only one, return it
+            return nets[0]
+        else:  # there are more, get the common CIDR
+            return str(spanning_cidr(nets))
 
-    def log_warn(self, msg):
-        self.log_msg(logging.WARNING, msg)
+    @staticmethod
+    def get_instance_selectors(system, mem_unit="M", disk_unit="M"):
+        cpu = 1
+        cpu_op_str = ">="
+        if system.getFeature('cpu.count'):
+            cpu = system.getValue('cpu.count')
+            cpu_op_str = system.getFeature('cpu.count').getLogOperator()
+        cpu_op = CloudConnector.OPERATORSMAP.get(cpu_op_str)
 
-    def log_exception(self, msg):
-        self.log_msg(logging.ERROR, msg, exc_info=1)
+        memory = 0
+        memory_op_str = ">="
+        if system.getFeature('memory.size'):
+            memory = system.getFeature('memory.size').getValue(mem_unit)
+            memory_op_str = system.getFeature('memory.size').getLogOperator()
+        memory_op = CloudConnector.OPERATORSMAP.get(memory_op_str)
 
-    def log_info(self, msg):
-        self.log_msg(logging.INFO, msg)
+        disk_free = 0
+        disk_free_op_str = ">="
+        if system.getValue('disks.free_size'):
+            disk_free = system.getFeature('disks.free_size').getValue(disk_unit)
+            disk_free_op_str = system.getFeature('disks.free_size').getLogOperator()
+        disk_free_op = CloudConnector.OPERATORSMAP.get(disk_free_op_str)
+
+        return (cpu, cpu_op, memory, memory_op, disk_free, disk_free_op)
+
+    @staticmethod
+    def cidr_wildcard_iterator(cidr, init=0):
+        """
+        Returns an interator with all the cidr nets that expand the wildcards passed.
+        For example: with cidr = 192.168.*.0/24
+        it will return:
+         - 192.168.1.1/24
+         - 192.168.2.1/24
+         - 192.168.3.1/24
+         - ...
+         - 192.168.253.1/24
+        """
+        if "*" in cidr:
+            for val in range(init, 253 + init):
+                val = val % 254 + 1
+                icidr = cidr.replace("*", str(val), 1)
+                if "*" in icidr:
+                    for elem in CloudConnector.cidr_wildcard_iterator(icidr):
+                        yield elem
+                else:
+                    yield icidr
+            cidr = icidr
+        else:
+            yield cidr
+
+    @staticmethod
+    def get_free_cidr(net_cidr, used_cidrs, inf=None, init=0):
+        """
+        Get a CIDR that is not used (is not in used_cidrs list)
+        """
+        if not net_cidr:
+            net_cidr = CloudConnector.DEFAULT_NET_CIDR
+
+        if "*" not in net_cidr:
+            return net_cidr
+
+        used_cidr_nets = [IPNetwork(net) for net in used_cidrs]
+
+        # add current used cidrs in other inf networks
+        if inf:
+            # Add the general RADL nets
+            for net in inf.radl.networks:
+                cidr = net.getValue('cidr')
+                if cidr and "*" not in cidr:
+                    used_cidr_nets.append(IPNetwork(cidr))
+
+            # and the nets from the VMs
+            # Use direct access to the list to avoid lock
+            for vm in inf.vm_list:
+                for net in vm.info.networks:
+                    print(net)
+                    cidr = net.getValue('cidr')
+                    if cidr and "*" not in cidr:
+                        used_cidr_nets.append(IPNetwork(cidr))
+
+        for cidr in CloudConnector.cidr_wildcard_iterator(net_cidr, init):
+            if not any([IPNetwork(cidr) in IPNetwork(mask) for mask in used_cidr_nets]):
+                return cidr
+
+        return None
