@@ -26,7 +26,10 @@ except Exception as ex:
     print(ex)
 
 from .LibCloud import LibCloudCloudConnector
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from radl.radl import Feature
 
@@ -58,11 +61,11 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         """
         auths = auth_data.getAuthInfo(self.type, self.cloud.server)
         if not auths:
-            raise Exception("No auth data has been specified to OpenStack.")
+            raise Exception("No auth data has been specified to CloudStack.")
         else:
             auth = auths[0]
 
-        if self.driver and self.auth.compare(auth_data, self.type):
+        if self.driver and self.auth.compare(auth_data, self.type, self.cloud.server):
             return self.driver
         else:
             self.auth = auth_data
@@ -85,64 +88,36 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
                 self.log_error("Incorrect auth data")
                 return None
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+        src_host = url[1].split(':')[0]
+
+        if protocol == "cst" and self.cloud.server == src_host:
+            driver = self.get_driver(auth_data)
+
+            res_system = radl_system.clone()
+            instance_type = self.get_instance_type(driver.list_sizes(), res_system)
+            self.update_system_info_from_instance(res_system, instance_type)
+
+            username = res_system.getValue('disk.0.os.credentials.username')
+            if not username:
+                res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
-
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-
-                src_host = url[1].split(':')[0]
-                # TODO: check the port
-                if protocol == "cst" and self.cloud.server == src_host:
-                    driver = self.get_driver(auth_data)
-
-                    res_system = radl_system.clone()
-                    instance_type = self.get_instance_type(driver.list_sizes(), res_system)
-                    self.update_system_info_from_instance(res_system, instance_type)
-
-                    res_system.addFeature(
-                        Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-
-                    res_system.addFeature(
-                        Feature("provider.type", "=", self.type), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.host", "=", self.cloud.server), conflict="other", missing="other")
-                    if self.cloud.port != -1:
-                        res_system.addFeature(Feature(
-                            "provider.port", "=", self.cloud.port), conflict="other", missing="other")
-
-                    username = res_system.getValue('disk.0.os.credentials.username')
-                    if not username:
-                        res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
-
-                    res.append(res_system)
-
-            return res
+            return None
 
     def update_system_info_from_instance(self, system, instance_type):
         """
         Update the features of the system with the information of the instance_type
         """
         if instance_type:
-            system.addFeature(Feature(
-                "memory.size", "=", instance_type.ram, 'M'), conflict="other", missing="other")
-            if instance_type.disk:
-                system.addFeature(Feature("disk.0.free_size", "=", instance_type.disk, 'G'),
-                                  conflict="other", missing="other")
+            LibCloudCloudConnector.update_system_info_from_instance(system, instance_type)
             if 'cpu' in instance_type.extra:
                 system.addFeature(Feature("cpu.count", "=", instance_type.extra['cpu']),
                                   conflict="other", missing="other")
 
-            system.addFeature(Feature("instance_type", "=",
-                                      instance_type.name), conflict="other", missing="other")
-
     def _get_security_group(self, driver, sg_name):
         try:
             sg = None
@@ -176,10 +151,10 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
 
             try:
                 # open always SSH port
-                success = driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
-                                                                     protocol='tcp',
-                                                                     startport=22,
-                                                                     cidrlist='0.0.0.0/0')
+                driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
+                                                           protocol='tcp',
+                                                           startport=22,
+                                                           cidrlist='0.0.0.0/0')
 
                 # open all the ports for the VMs in the security group
 #                 usersecuritygrouplist = [{'group': sg['name'], 'account': sg['account'],}]
@@ -204,20 +179,20 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
                 for op in outports:
                     if op.is_range():
                         try:
-                            success = driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
-                                                                                 protocol=op.get_protocol(),
-                                                                                 startport=op.get_port_init(),
-                                                                                 endport=op.get_port_end(),
-                                                                                 cidrlist='0.0.0.0/0')
+                            driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
+                                                                       protocol=op.get_protocol(),
+                                                                       startport=op.get_port_init(),
+                                                                       endport=op.get_port_end(),
+                                                                       cidrlist='0.0.0.0/0')
                         except Exception as ex:
                             self.log_warn("Exception adding SG rules: " + str(ex))
                     else:
                         if op.get_remote_port() != 22:
                             try:
-                                success = driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
-                                                                                     protocol=op.get_protocol(),
-                                                                                     startport=op.get_remote_port(),
-                                                                                     cidrlist='0.0.0.0/0')
+                                driver.ex_authorize_security_group_ingress(securitygroupname=sg_name,
+                                                                           protocol=op.get_protocol(),
+                                                                           startport=op.get_remote_port(),
+                                                                           cidrlist='0.0.0.0/0')
                             except Exception as ex:
                                 self.log_warn("Exception adding SG rules: " + str(ex))
 
@@ -245,6 +220,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         args = {'size': instance_type,
                 'image': image,
                 'ex_security_groups': sgs,
+                'ex_start_vm': True,
                 'name': "%s-%s" % (name, str(uuid.uuid1()))}
 
         if system.getValue('availability_zone'):
@@ -269,14 +245,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
             user = self.DEFAULT_USER
             system.setValue('disk.0.os.credentials.username', user)
 
-        tags = {}
-        if system.getValue('instance_tags'):
-            keypairs = system.getValue('instance_tags').split(",")
-            for keypair in keypairs:
-                parts = keypair.split("=")
-                key = parts[0].strip()
-                value = parts[1].strip()
-                tags[key] = value
+        tags = self.get_instance_tags(system)
 
         res = []
         i = 0
@@ -291,9 +260,11 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
             if cloud_init:
                 args['ex_userdata'] = cloud_init
 
+            msg = "Error creating the node"
             try:
                 node = driver.create_node(**args)
-            except:
+            except Exception as ex:
+                msg += ": %s" % str(ex)
                 self.log_exception("Error creating node.")
                 node = None
 
@@ -301,7 +272,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
                 if tags:
                     try:
                         driver.ex_create_tags([node.id], tags)
-                    except:
+                    except Exception:
                         self.log_exception("Error adding tags to node %s." % node.id)
 
                 vm.id = node.id
@@ -314,7 +285,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
                 inf.add_vm(vm)
                 res.append((True, vm))
             else:
-                res.append((False, "Error creating the node"))
+                res.append((False, msg))
 
             i += 1
 
@@ -381,7 +352,7 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
             else:
                 # If this is not the last vm, we skip this step
                 self.log_info("There are active instances. Not removing the SG")
-        except:
+        except Exception:
             self.log_exception("Error deleting security groups.")
 
         return (True, "")
@@ -440,6 +411,17 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         else:
             return (False, "VM not found with id: " + vm.id)
 
+    def reboot(self, vm, auth_data):
+        node = self.get_node_with_id(vm.id, auth_data)
+        if node:
+            success = node.reboot_node()
+            if success:
+                return (True, "")
+            else:
+                return (False, "Error in reboot operation")
+        else:
+            return (False, "VM not found with id: " + vm.id)
+
     def alterVM(self, vm, radl, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
@@ -477,29 +459,6 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         else:
             return (False, "VM not found with id: " + vm.id)
 
-    def wait_volume(self, volume, state='available', timeout=60):
-        """
-        Wait a volume (with the state extra parameter) to be in certain state.
-
-        Arguments:
-           - volume(:py:class:`libcloud.compute.base.StorageVolume`): volume object or boolean.
-           - state(str): State to wait for (default value 'available').
-           - timeout(int): Max time to wait in seconds (default value 60).
-        """
-        if volume:
-            if 'state' in volume.extra:
-                cont = 0
-                err_states = ["error"]
-                while volume.extra['state'] != state and volume.extra['state'] not in err_states and cont < timeout:
-                    cont += 2
-                    time.sleep(2)
-                    volume = volume.driver.ex_get_volume(volume.id)
-                return volume.extra['state'] == state
-
-            return True
-        else:
-            return False
-
     def attach_volumes(self, vm, node):
         """
         Attach a the required volumes (in the RADL) to the launched node
@@ -545,32 +504,3 @@ class CloudStackCloudConnector(LibCloudCloudConnector):
         except Exception:
             self.log_exception("Error creating or attaching the volume to the node")
             return False
-
-    def delete_volumes(self, driver, vm, timeout=300):
-        """
-        Delete the volumes of a VM
-
-        Arguments:
-           - vm(:py:class:`IM.VirtualMachine`): VM information.
-           - timeout(int): Time needed to delete the volume.
-        """
-        all_ok = True
-        if "volumes" in vm.__dict__.keys() and vm.volumes:
-            for volumeid in vm.volumes:
-                self.log_debug("Deleting volume ID %s" % volumeid)
-                try:
-                    volume = driver.ex_get_volume(volumeid)
-                    success = self.wait_volume(volume, timeout=timeout)
-                    if not success:
-                        self.log_error("Error waiting the volume ID " + str(volume.id))
-                    success = volume.destroy()
-                    if not success:
-                        self.log_error("Error destroying the volume: " + str(volume.id))
-                except:
-                    self.log_exception("Error destroying the volume: " + str(volume.id) +
-                                       " from the node: " + str(vm.id))
-                    success = False
-
-                if not success:
-                    all_ok = False
-        return all_ok
