@@ -117,7 +117,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                           "image_url": None,
                           "volume_url": None,
                           "api_version": "2.0",
-                          "domain": None}
+                          "domain": None,
+                          "tenant_domain_id": None}
 
             if 'username' in auth and 'password' in auth and 'tenant' in auth:
                 username = auth['username']
@@ -161,6 +162,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                         key = 'ex_force_%s' % key
                     elif key == 'domain':
                         key = 'ex_domain_name'
+                    elif key == 'tenant_domain_id':
+                        key = 'ex_tenant_domain_id'
                     kwargs[key] = value
 
             # Workaround to OTC to enable to set service_name as None
@@ -1301,6 +1304,23 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         success = []
         msgs = []
         if node:
+            # First try to detach the volumes and the SGs
+            for vol_id in vm.volumes:
+                try:
+                    self.log_debug("Dettaching volume %s." % vol_id)
+                    volume = node.driver.ex_get_volume(vol_id)
+                    node.driver.detach_volume(volume)
+                except Exception as ex:
+                    self.log_exception("Error dettaching volume %s." % vol_id)
+
+            try:
+                for sg_name in self._get_security_names(vm.inf):
+                    self.log_debug("Dettaching SG %s." % sg_name)
+                    security_group = OpenStackSecurityGroup(None, None, sg_name, "", node.driver)
+                    node.driver.ex_remove_security_group_from_node(security_group, node)
+            except Exception as ex:
+                self.log_exception("Error dettaching SG %s." % sg_name)
+
             res = node.destroy()
             success.append(res)
 
@@ -1358,9 +1378,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return (all(success), "\n ".join(msgs))
 
-    def delete_security_groups(self, driver, inf, timeout=180, delay=10):
+    def _get_security_names(self, inf):
         """
-        Delete the SG of this inf
+        Get the list of SGs for this infra
         """
         sg_names = ["im-%s" % inf.id]
         for net in inf.radl.networks:
@@ -1369,6 +1389,14 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 sg_name = "im-%s-%s" % (inf.id, net.id)
             sg_names.append(sg_name)
 
+        return sg_names
+
+    def delete_security_groups(self, driver, inf, timeout=180, delay=10):
+        """
+        Delete the SG of this inf
+        """
+        sg_names = self._get_security_names(inf)
+
         msg = ""
         deleted = True
         for sg_name in sg_names:
