@@ -17,13 +17,16 @@
 """Class to manage DB operations"""
 import time
 
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 
 try:
     import sqlite3 as sqlite
     SQLITE3_AVAILABLE = True
     SQLITE_AVAILABLE = True
-except:
+except Exception:
     SQLITE3_AVAILABLE = False
     SQLITE_AVAILABLE = False
 
@@ -31,30 +34,37 @@ if not SQLITE_AVAILABLE:
     try:
         import sqlite
         SQLITE_AVAILABLE = True
-    except:
+    except Exception:
         SQLITE_AVAILABLE = False
 
 try:
     import MySQLdb as mdb
     MYSQL_AVAILABLE = True
-except:
+except Exception:
     MYSQL_AVAILABLE = False
 
 if not MYSQL_AVAILABLE:
     try:
         import pymysql as mdb
         MYSQL_AVAILABLE = True
-    except:
+    except Exception:
         MYSQL_AVAILABLE = False
 
+try:
+    from pymongo import MongoClient
+    MONGO_AVAILABLE = True
+except Exception:
+    MONGO_AVAILABLE = False
+
 
 # Class to manage DB operations
 class DataBase:
     """Class to manage DB operations"""
 
-    db_available = SQLITE_AVAILABLE or MYSQL_AVAILABLE
+    db_available = SQLITE_AVAILABLE or MYSQL_AVAILABLE or MONGO_AVAILABLE
     RETRY_SLEEP = 2
     MAX_RETRIES = 15
+    MONGO = "MONGO"
     MYSQL = "MySQL"
     SQLITE = "SQLite"
     DB_TYPES = [MYSQL, SQLITE]
@@ -70,8 +80,10 @@ class DataBase:
             Returns: True if the connection is established correctly
                      of False in case of errors.
         """
-        uri = uriparse(self.db_url)
+        uri = urlparse(self.db_url)
         protocol = uri[0]
+        if protocol == "mongodb":
+            return self._connect_mongo(uri[0] + "://" + uri[1], uri[2][1:])
         if protocol == "mysql":
             return self._connect_mysql(uri[1], uri[2][1:])
         elif protocol == "file" or protocol == "sqlite" or not protocol:  # sqlite is the default one
@@ -79,8 +91,11 @@ class DataBase:
 
         return False
 
-    def _get_user_pass_host_port(self, url):
-        username = password = server = port = None
+    @staticmethod
+    def _get_user_pass_host_port(url):
+        username = None
+        password = None
+        port = None
         if "@" in url:
             parts = url.split("@")
             user_pass = parts[0]
@@ -99,6 +114,15 @@ class DataBase:
 
         return username, password, server, port
 
+    def _connect_mongo(self, url, db):
+        if MONGO_AVAILABLE:
+            client = MongoClient(url)
+            self.connection = client[db]
+            self.db_type = DataBase.MONGO
+            return True
+        else:
+            return False
+
     def _connect_mysql(self, url, db):
         if MYSQL_AVAILABLE:
             username, password, server, port = self._get_user_pass_host_port(url)
@@ -177,6 +201,8 @@ class DataBase:
 
             Returns: True if the operation is performed correctly
         """
+        if self.db_type == DataBase.MONGO:
+            raise Exception("Operation not supported in MongoDB")
         return self._execute_retry(sql, args)
 
     def select(self, sql, args=None):
@@ -189,6 +215,8 @@ class DataBase:
 
             Returns: A list with the "Fetch" of the results
         """
+        if self.db_type == DataBase.MONGO:
+            raise Exception("Operation not supported in MongoDB")
         return self._execute_retry(sql, args, fetch=True)
 
     def close(self):
@@ -197,7 +225,10 @@ class DataBase:
             return False
         else:
             try:
-                self.connection.close()
+                if self.db_type == DataBase.MONGO:
+                    self.connection.client.close()
+                else:
+                    self.connection.close()
                 return True
             except Exception:
                 return False
@@ -210,14 +241,15 @@ class DataBase:
 
             Returns: True if the table exists or False otherwise
         """
-        if self.db_type == "SQLite":
-            res = self.select(
-                'select name from sqlite_master where type="table" and name="' + table_name + '"')
-        elif self.db_type == "MySQL":
-            uri = uriparse(self.db_url)
+        if self.db_type == DataBase.SQLITE:
+            res = self.select('select name from sqlite_master where type="table" and name= %s', (table_name,))
+        elif self.db_type == DataBase.MYSQL:
+            uri = urlparse(self.db_url)
             db = uri[2][1:]
-            res = self.select('SELECT * FROM information_schema.tables WHERE table_name ="' +
-                              table_name + '" and table_schema = "' + db + '"')
+            res = self.select('SELECT * FROM information_schema.tables WHERE table_name = %s and table_schema = %s',
+                              (table_name, db))
+        elif self.db_type == DataBase.MONGO:
+            return table_name in self.connection.collection_names()
         else:
             return False
 
@@ -226,11 +258,44 @@ class DataBase:
         else:
             return True
 
+    def find(self, table_name, filt=None, projection=None, sort=None):
+        """ find elements """
+        if self.db_type != DataBase.MONGO:
+            raise Exception("Operation only supported in MongoDB")
+
+        if self.connection is None:
+            raise Exception("DataBase object not connected")
+        else:
+            if projection:
+                projection.update({'_id': False})
+            return list(self.connection[table_name].find(filt, projection, sort=sort))
+
+    def replace(self, table_name, filt, replacement):
+        """ insert/replace elements """
+        if self.db_type != DataBase.MONGO:
+            raise Exception("Operation only supported in MongoDB")
+
+        if self.connection is None:
+            raise Exception("DataBase object not connected")
+        else:
+            res = self.connection[table_name].replace_one(filt, replacement, True)
+            return res.modified_count == 1 or res.upserted_id is not None
+
+    def delete(self, table_name, filt):
+        """ delete elements """
+        if self.db_type != DataBase.MONGO:
+            raise Exception("Operation only supported in MongoDB")
+
+        if self.connection is None:
+            raise Exception("DataBase object not connected")
+        else:
+            return self.connection[table_name].delete_many(filt).deleted_count
+
 
 try:
     class IntegrityError(sqlite.IntegrityError):
         """ Class to return IntegrityError independently of the DB used"""
         pass
-except:
+except Exception:
     class IntegrityError:
         pass
